"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.flattenToSelectors = function(cssAst) {
  const selectors = [];
  _cssTree.default.walk(cssAst, {
    visit: "Rule",
    enter: function(node) {
      if ("Rule" !== node.type) return;
      const atrule = this.atrule;
      const rule = node;
      node.prelude.children.each((function(selectorNode, selectorItem) {
        const selector = {
          item: selectorItem,
          atrule,
          rule,
          pseudos: []
        };
        selectorNode.children.each((function(selectorChildNode, selectorChildItem, selectorChildList) {
          "PseudoClassSelector" !== selectorChildNode.type && "PseudoElementSelector" !== selectorChildNode.type || selector.pseudos.push({
            item: selectorChildItem,
            list: selectorChildList
          });
        }));
        selectors.push(selector);
      }));
    }
  });
  return selectors;
};

exports.filterByMqs = function(selectors, useMqs) {
  return selectors.filter((function(selector) {
    if (null === selector.atrule) return ~useMqs.indexOf("");
    const mqName = selector.atrule.name;
    let mqStr = mqName;
    selector.atrule.expression && "MediaQueryList" === selector.atrule.expression.children.first().type && (mqStr = [ mqName, _cssTree.default.generate(selector.atrule.expression) ].join(" "));
    return ~useMqs.indexOf(mqStr);
  }));
};

exports.filterByPseudos = function(selectors, usePseudos) {
  return selectors.filter((function(selector) {
    const pseudoSelectorsStr = _cssTree.default.generate({
      type: "Selector",
      children: (new List).fromArray(selector.pseudos.map((function(pseudo) {
        return pseudo.item.data;
      })))
    });
    return ~usePseudos.indexOf(pseudoSelectorsStr);
  }));
};

exports.cleanPseudos = function(selectors) {
  selectors.forEach((function(selector) {
    selector.pseudos.forEach((function(pseudo) {
      pseudo.list.remove(pseudo.item);
    }));
  }));
};

exports.compareSpecificity = compareSpecificity;

exports.compareSimpleSelectorNode = compareSimpleSelectorNode;

exports.sortSelectors = function(selectors) {
  return (0, _stable.default)(selectors, _bySelectorSpecificity);
};

exports.csstreeToStyleDeclaration = function(declaration) {
  return {
    name: declaration.property,
    value: _cssTree.default.generate(declaration.value),
    priority: declaration.important ? "important" : ""
  };
};

exports.getCssStr = function(elem) {
  return elem.content[0].text || elem.content[0].cdata || [];
};

exports.setCssStr = function(elem, css) {
  if (elem.content[0].cdata) {
    elem.content[0].cdata = css;
    return elem.content[0].cdata;
  }
  elem.content[0].text = css;
  return elem.content[0].text;
};

var _stable = _interopRequireDefault(require("stable"));

var _specificity = _interopRequireDefault(require("csso/lib/restructure/prepare/specificity"));

var _cssTree = _interopRequireDefault(require("css-tree"));

function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

const List = _cssTree.default.List;

function compareSpecificity(aSpecificity, bSpecificity) {
  for (let i = 0; i < 4; i += 1) {
    if (aSpecificity[i] < bSpecificity[i]) return -1;
    if (aSpecificity[i] > bSpecificity[i]) return 1;
  }
  return 0;
}

function compareSimpleSelectorNode(aSimpleSelectorNode, bSimpleSelectorNode) {
  return compareSpecificity((0, _specificity.default)(aSimpleSelectorNode), (0, _specificity.default)(bSimpleSelectorNode));
}

function _bySelectorSpecificity(selectorA, selectorB) {
  return compareSimpleSelectorNode(selectorA.item.data, selectorB.item.data);
}