"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.SVGO_LITE = void 0;

var _configLite = require("./svgo/config-lite");

var _svg2js = require("./svgo/svg2js");

var _plugins = require("./svgo/plugins");

var _jsAPI = require("./svgo/jsAPI");

var _js2svg = require("./svgo/js2svg");

var _tools = require("./svgo/tools");

/**
 * SVGO is a Nodejs-based tool for optimizing SVG vector graphics files.
 *
 * @see https://github.com/svg/svgo
 *
 * @author Kir Belevich <kir@soulshine.in> (https://github.com/deepsweet)
 * @copyright © 2012 Kir Belevich
 * @license MIT https://raw.githubusercontent.com/svg/svgo/master/LICENSE
 */ const SVGO_LITE = function(config) {
  this.config = SVGO_LITE.Config(config);
};

exports.SVGO_LITE = SVGO_LITE;

SVGO_LITE.Config = _configLite.CONFIG_LITE;

SVGO_LITE.prototype.optimize = function(svgstr, info) {
  info = info || {};
  return new Promise(((resolve, reject) => {
    if (this.config.error) {
      reject(this.config.error);
      return;
    }
    const config = this.config;
    const maxPassCount = config.multipass ? 10 : 1;
    let counter = 0;
    let prevResultSize = Number.POSITIVE_INFINITY;
    const optimizeOnceCallback = svgjs => {
      if (svgjs.error) reject(svgjs.error); else {
        info.multipassCount = counter;
        if (++counter < maxPassCount && svgjs.data.length < prevResultSize) {
          prevResultSize = svgjs.data.length;
          this._optimizeOnce(svgjs.data, info, optimizeOnceCallback);
        } else {
          config.datauri && (svgjs.data = (0, _tools.encodeSVGDatauri)(svgjs.data, config.datauri));
          info && info.path && (svgjs.path = info.path);
          resolve(svgjs);
        }
      }
    };
    this._optimizeOnce(svgstr, info, optimizeOnceCallback);
  }));
};

SVGO_LITE.prototype._optimizeOnce = function(svgstr, info, callback) {
  const config = this.config;
  (0, _svg2js.SVG2JS)(svgstr, (function(svgjs) {
    if (svgjs.error) callback(svgjs); else {
      svgjs = (0, _plugins.PLUGINS)(svgjs, info, config.plugins);
      callback((0, _js2svg.JS2SVG)(svgjs, config.js2svg));
    }
  }));
};

SVGO_LITE.prototype.createContentItem = function(data) {
  return new _jsAPI.JSAPI(data);
};