"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.CSSClassList = void 0;

const CSSClassList = function(node) {
  this.parentNode = node;
  this.classNames = new Set;
  this.classAttr = null;
};

exports.CSSClassList = CSSClassList;

CSSClassList.prototype.clone = function(parentNode) {
  const node = this;
  let nodeData = {};
  Object.keys(node).forEach((function(key) {
    "parentNode" !== key && (nodeData[key] = node[key]);
  }));
  nodeData = JSON.parse(JSON.stringify(nodeData));
  const clone = new CSSClassList(parentNode);
  Object.assign(clone, nodeData);
  return clone;
};

CSSClassList.prototype.hasClass = function() {
  this.classAttr = {
    name: "class",
    value: null
  };
  this.addClassHandler();
};

CSSClassList.prototype.addClassHandler = function() {
  Object.defineProperty(this.parentNode.attrs, "class", {
    get: this.getClassAttr.bind(this),
    set: this.setClassAttr.bind(this),
    enumerable: !0,
    configurable: !0
  });
  this.addClassValueHandler();
};

CSSClassList.prototype.addClassValueHandler = function() {
  Object.defineProperty(this.classAttr, "value", {
    get: this.getClassValue.bind(this),
    set: this.setClassValue.bind(this),
    enumerable: !0,
    configurable: !0
  });
};

CSSClassList.prototype.getClassAttr = function() {
  return this.classAttr;
};

CSSClassList.prototype.setClassAttr = function(newClassAttr) {
  this.setClassValue(newClassAttr.value);
  this.classAttr = newClassAttr;
  this.addClassValueHandler();
};

CSSClassList.prototype.getClassValue = function() {
  return Array.from(this.classNames).join(" ");
};

CSSClassList.prototype.setClassValue = function(newValue) {
  if (void 0 === newValue) {
    this.classNames.clear();
    return;
  }
  const arrClassNames = newValue.split(" ");
  this.classNames = new Set(arrClassNames);
};

CSSClassList.prototype.add = function() {
  this.hasClass();
  Object.values(arguments).forEach(this._addSingle.bind(this));
};

CSSClassList.prototype._addSingle = function(className) {
  this.classNames.add(className);
};

CSSClassList.prototype.remove = function() {
  this.hasClass();
  Object.values(arguments).forEach(this._removeSingle.bind(this));
};

CSSClassList.prototype._removeSingle = function(className) {
  this.classNames.delete(className);
};

CSSClassList.prototype.item = function(index) {
  return Array.from(this.classNames)[index];
};

CSSClassList.prototype.toggle = function(className, force) {
  (this.contains(className) || !1 === force) && this.classNames.delete(className);
  this.classNames.add(className);
};

CSSClassList.prototype.contains = function(className) {
  return this.classNames.has(className);
};