"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.CSSStyleDeclaration = void 0;

var _cssTree = (obj = require("css-tree")) && obj.__esModule ? obj : {
  default: obj
};

var obj;

var _cssTools = require("../css-tools");

const CSSStyleDeclaration = function(node) {
  this.parentNode = node;
  this.properties = new Map;
  this.hasSynced = !1;
  this.styleAttr = null;
  this.styleValue = null;
  this.parseError = !1;
};

exports.CSSStyleDeclaration = CSSStyleDeclaration;

CSSStyleDeclaration.prototype.clone = function(parentNode) {
  var node = this;
  var nodeData = {};
  Object.keys(node).forEach((function(key) {
    "parentNode" !== key && (nodeData[key] = node[key]);
  }));
  nodeData = JSON.parse(JSON.stringify(nodeData));
  var clone = new CSSStyleDeclaration(parentNode);
  Object.assign(clone, nodeData);
  return clone;
};

CSSStyleDeclaration.prototype.hasStyle = function() {
  this.addStyleHandler();
};

CSSStyleDeclaration.prototype.addStyleHandler = function() {
  this.styleAttr = {
    name: "style",
    value: null
  };
  Object.defineProperty(this.parentNode.attrs, "style", {
    get: this.getStyleAttr.bind(this),
    set: this.setStyleAttr.bind(this),
    enumerable: !0,
    configurable: !0
  });
  this.addStyleValueHandler();
};

CSSStyleDeclaration.prototype.addStyleValueHandler = function() {
  Object.defineProperty(this.styleAttr, "value", {
    get: this.getStyleValue.bind(this),
    set: this.setStyleValue.bind(this),
    enumerable: !0,
    configurable: !0
  });
};

CSSStyleDeclaration.prototype.getStyleAttr = function() {
  return this.styleAttr;
};

CSSStyleDeclaration.prototype.setStyleAttr = function(newStyleAttr) {
  this.setStyleValue(newStyleAttr.value);
  this.styleAttr = newStyleAttr;
  this.addStyleValueHandler();
  this.hasSynced = !1;
};

CSSStyleDeclaration.prototype.getStyleValue = function() {
  return this.getCssText();
};

CSSStyleDeclaration.prototype.setStyleValue = function(newValue) {
  this.properties.clear();
  this.styleValue = newValue;
  this.hasSynced = !1;
};

CSSStyleDeclaration.prototype._loadCssText = function() {
  if (!this.hasSynced) {
    this.hasSynced = !0;
    if (this.styleValue && 0 !== this.styleValue.length) {
      var inlineCssStr = this.styleValue;
      var declarations = {};
      try {
        declarations = _cssTree.default.parse(inlineCssStr, {
          context: "declarationList",
          parseValue: !1
        });
      } catch (parseError) {
        this.parseError = parseError;
        return;
      }
      this.parseError = !1;
      var self = this;
      declarations.children.each((function(declaration) {
        try {
          var styleDeclaration = (0, _cssTools.csstreeToStyleDeclaration)(declaration);
          self.setProperty(styleDeclaration.name, styleDeclaration.value, styleDeclaration.priority);
        } catch (styleError) {
          "Unknown node type: undefined" !== styleError.message && (self.parseError = styleError);
        }
      }));
    }
  }
};

CSSStyleDeclaration.prototype.getCssText = function() {
  var properties = this.getProperties();
  if (this.parseError) return this.styleValue;
  var cssText = [];
  properties.forEach((function(property, propertyName) {
    var strImportant = "important" === property.priority ? "!important" : "";
    cssText.push(propertyName.trim() + ":" + property.value.trim() + strImportant);
  }));
  return cssText.join(";");
};

CSSStyleDeclaration.prototype._handleParseError = function() {
  this.parseError && console.warn("Warning: Parse error when parsing inline styles, style properties of this element cannot be used. The raw styles can still be get/set using .attr('style').value. Error details: " + this.parseError);
};

CSSStyleDeclaration.prototype._getProperty = function(propertyName) {
  if (void 0 === propertyName) throw Error("1 argument required, but only 0 present.");
  var properties = this.getProperties();
  this._handleParseError();
  return properties.get(propertyName.trim());
};

CSSStyleDeclaration.prototype.getPropertyPriority = function(propertyName) {
  var property = this._getProperty(propertyName);
  return property ? property.priority : "";
};

CSSStyleDeclaration.prototype.getPropertyValue = function(propertyName) {
  var property = this._getProperty(propertyName);
  return property ? property.value : null;
};

CSSStyleDeclaration.prototype.item = function(index) {
  if (void 0 === index) throw Error("1 argument required, but only 0 present.");
  var properties = this.getProperties();
  this._handleParseError();
  return Array.from(properties.keys())[index];
};

CSSStyleDeclaration.prototype.getProperties = function() {
  this._loadCssText();
  return this.properties;
};

CSSStyleDeclaration.prototype.removeProperty = function(propertyName) {
  if (void 0 === propertyName) throw Error("1 argument required, but only 0 present.");
  this.hasStyle();
  var properties = this.getProperties();
  this._handleParseError();
  var oldValue = this.getPropertyValue(propertyName);
  properties.delete(propertyName.trim());
  return oldValue;
};

CSSStyleDeclaration.prototype.setProperty = function(propertyName, value, priority) {
  if (void 0 === propertyName) throw Error("propertyName argument required, but only not present.");
  this.hasStyle();
  var properties = this.getProperties();
  this._handleParseError();
  var property = {
    value: value.trim(),
    priority: priority.trim()
  };
  properties.set(propertyName.trim(), property);
  return property;
};