"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.JS2SVG = void 0;

const textElem = require("../../plugins/_collections").elemsGroups.textContent.concat("title");

const defaults = {
  doctypeStart: "<!DOCTYPE",
  doctypeEnd: ">",
  procInstStart: "<?",
  procInstEnd: "?>",
  tagOpenStart: "<",
  tagOpenEnd: ">",
  tagCloseStart: "</",
  tagCloseEnd: ">",
  tagShortStart: "<",
  tagShortEnd: "/>",
  attrStart: '="',
  attrEnd: '"',
  commentStart: "\x3c!--",
  commentEnd: "--\x3e",
  cdataStart: "<![CDATA[",
  cdataEnd: "]]>",
  textStart: "",
  textEnd: "",
  indent: 4,
  regEntities: /[&'"<>]/g,
  regValEntities: /[&"<>]/g,
  encodeEntity: function(char) {
    return entities[char];
  },
  pretty: !1,
  useShortTags: !0
};

const entities = {
  "&": "&amp;",
  "'": "&apos;",
  '"': "&quot;",
  ">": "&gt;",
  "<": "&lt;"
};

exports.JS2SVG = function(data, config) {
  return new JS2SVG(config).convert(data);
};

function JS2SVG(config) {
  this.config = config ? Object.assign({}, defaults, config) : Object.assign({}, defaults);
  const indent = this.config.indent;
  "number" != typeof indent || isNaN(indent) ? "string" != typeof indent && (this.config.indent = "    ") : this.config.indent = indent < 0 ? "\t" : " ".repeat(indent);
  if (this.config.pretty) {
    this.config.doctypeEnd += "\n";
    this.config.procInstEnd += "\n";
    this.config.commentEnd += "\n";
    this.config.cdataEnd += "\n";
    this.config.tagShortEnd += "\n";
    this.config.tagOpenEnd += "\n";
    this.config.tagCloseEnd += "\n";
    this.config.textEnd += "\n";
  }
  this.indentLevel = 0;
  this.textContext = null;
}

JS2SVG.prototype.convert = function(data) {
  let svg = "";
  if (data.content) {
    this.indentLevel++;
    data.content.forEach((function(item) {
      item.elem ? svg += this.createElem(item) : item.text ? svg += this.createText(item.text) : item.doctype ? svg += this.createDoctype(item.doctype) : item.processinginstruction ? svg += this.createProcInst(item.processinginstruction) : item.comment ? svg += this.createComment(item.comment) : item.cdata && (svg += this.createCDATA(item.cdata));
    }), this);
  }
  this.indentLevel--;
  return {
    data: svg,
    info: {
      width: this.width,
      height: this.height
    }
  };
};

JS2SVG.prototype.createIndent = function() {
  let indent = "";
  this.config.pretty && !this.textContext && (indent = this.config.indent.repeat(this.indentLevel - 1));
  return indent;
};

JS2SVG.prototype.createDoctype = function(doctype) {
  return this.config.doctypeStart + doctype + this.config.doctypeEnd;
};

JS2SVG.prototype.createProcInst = function(instruction) {
  return this.config.procInstStart + instruction.name + " " + instruction.body + this.config.procInstEnd;
};

JS2SVG.prototype.createComment = function(comment) {
  return this.config.commentStart + comment + this.config.commentEnd;
};

JS2SVG.prototype.createCDATA = function(cdata) {
  return this.createIndent() + this.config.cdataStart + cdata + this.config.cdataEnd;
};

JS2SVG.prototype.createElem = function(data) {
  if (data.isElem("svg") && data.hasAttr("width") && data.hasAttr("height")) {
    this.width = data.attr("width").value;
    this.height = data.attr("height").value;
  }
  if (data.isEmpty()) return this.config.useShortTags ? this.createIndent() + this.config.tagShortStart + data.elem + this.createAttrs(data) + this.config.tagShortEnd : this.createIndent() + this.config.tagShortStart + data.elem + this.createAttrs(data) + this.config.tagOpenEnd + this.config.tagCloseStart + data.elem + this.config.tagCloseEnd;
  {
    let tagOpenStart = this.config.tagOpenStart;
    let tagOpenEnd = this.config.tagOpenEnd;
    let tagCloseStart = this.config.tagCloseStart;
    let tagCloseEnd = this.config.tagCloseEnd;
    let openIndent = this.createIndent();
    let textIndent = "";
    let processedData = "";
    let dataEnd = "";
    if (this.textContext) {
      tagOpenStart = defaults.tagOpenStart;
      tagOpenEnd = defaults.tagOpenEnd;
      tagCloseStart = defaults.tagCloseStart;
      tagCloseEnd = defaults.tagCloseEnd;
      openIndent = "";
    } else if (data.isElem(textElem)) {
      this.config.pretty && (textIndent += openIndent + this.config.indent);
      this.textContext = data;
    }
    processedData += this.convert(data).data;
    if (this.textContext === data) {
      this.textContext = null;
      this.config.pretty && (dataEnd = "\n");
    }
    return openIndent + tagOpenStart + data.elem + this.createAttrs(data) + tagOpenEnd + textIndent + processedData + dataEnd + this.createIndent() + tagCloseStart + data.elem + tagCloseEnd;
  }
};

JS2SVG.prototype.createAttrs = function(elem) {
  let attrs = "";
  elem.eachAttr((function(attr) {
    void 0 !== attr.value ? attrs += " " + attr.name + this.config.attrStart + String(attr.value).replace(this.config.regValEntities, this.config.encodeEntity) + this.config.attrEnd : attrs += " " + attr.name;
  }), this);
  return attrs;
};

JS2SVG.prototype.createText = function(text) {
  return this.createIndent() + this.config.textStart + text.replace(this.config.regEntities, this.config.encodeEntity) + (this.textContext ? "" : this.config.textEnd);
};