"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.JSAPI = void 0;

var _cssSelect = require("css-select");

const cssSelectOpts = {
  xmlMode: !0,
  adapter: require("./css-select-adapter").svgoCssSelectAdapter
};

const JSAPI = function(data, parentNode) {
  Object.assign(this, data);
  parentNode && Object.defineProperty(this, "parentNode", {
    writable: !0,
    value: parentNode
  });
};

exports.JSAPI = JSAPI;

JSAPI.prototype.clone = function() {
  const node = this;
  let nodeData = {};
  Object.keys(node).forEach((function(key) {
    "class" !== key && "style" !== key && "content" !== key && (nodeData[key] = node[key]);
  }));
  nodeData = JSON.parse(JSON.stringify(nodeData));
  const clonedNode = new JSAPI(nodeData, !!node.parentNode);
  node.class && (clonedNode.class = node.class.clone(clonedNode));
  node.style && (clonedNode.style = node.style.clone(clonedNode));
  node.content && (clonedNode.content = node.content.map((function(childNode) {
    const clonedChild = childNode.clone();
    clonedChild.parentNode = clonedNode;
    return clonedChild;
  })));
  return clonedNode;
};

JSAPI.prototype.isElem = function(param) {
  return param ? Array.isArray(param) ? !!this.elem && param.indexOf(this.elem) > -1 : !!this.elem && this.elem === param : !!this.elem;
};

JSAPI.prototype.renameElem = function(name) {
  name && "string" == typeof name && (this.elem = this.local = name);
  return this;
};

JSAPI.prototype.isEmpty = function() {
  return !this.content || !this.content.length;
};

JSAPI.prototype.closestElem = function(elemName) {
  let elem = this;
  for (;(elem = elem.parentNode) && !elem.isElem(elemName); ) ;
  return elem;
};

JSAPI.prototype.spliceContent = function(start, n, insertion) {
  if (arguments.length < 2) return [];
  Array.isArray(insertion) || (insertion = Array.apply(null, arguments).slice(2));
  insertion.forEach((function(inner) {
    inner.parentNode = this;
  }), this);
  return this.content.splice.apply(this.content, [ start, n ].concat(insertion));
};

JSAPI.prototype.hasAttr = function(name, val) {
  return !!(this.attrs && Object.keys(this.attrs).length && (arguments.length ? void 0 !== val ? this.attrs[name] && this.attrs[name].value === val.toString() : this.attrs[name] : this.attrs));
};

JSAPI.prototype.hasAttrLocal = function(localName, val) {
  if (!this.attrs || !Object.keys(this.attrs).length) return !1;
  if (!arguments.length) return !!this.attrs;
  let callback;
  switch (null != val && val.constructor && val.constructor.name) {
   case "Number":
   case "String":
    callback = stringValueTest;
    break;

   case "RegExp":
    callback = regexpValueTest;
    break;

   case "Function":
    callback = funcValueTest;
    break;

   default:
    callback = nameTest;
  }
  return this.someAttr(callback);
  function nameTest(attr) {
    return attr.local === localName;
  }
  function stringValueTest(attr) {
    return attr.local === localName && val === attr.value;
  }
  function regexpValueTest(attr) {
    return attr.local === localName && val.test(attr.value);
  }
  function funcValueTest(attr) {
    return attr.local === localName && val(attr.value);
  }
};

JSAPI.prototype.attr = function(name, val) {
  if (this.hasAttr() && arguments.length) return void 0 !== val ? this.hasAttr(name, val) ? this.attrs[name] : void 0 : this.attrs[name];
};

JSAPI.prototype.computedAttr = function(name, val) {
  if (!arguments.length) return;
  let elem = this;
  for (;elem && (!elem.hasAttr(name) || !elem.attr(name).value); elem = elem.parentNode) ;
  return null != val ? !!elem && elem.hasAttr(name, val) : elem && elem.hasAttr(name) ? elem.attrs[name].value : void 0;
};

JSAPI.prototype.removeAttr = function(name, val, recursive) {
  if (!arguments.length) return !1;
  if (Array.isArray(name)) {
    name.forEach(this.removeAttr, this);
    return !1;
  }
  if (!this.hasAttr(name)) return !1;
  if (!recursive && val && this.attrs[name].value !== val) return !1;
  delete this.attrs[name];
  Object.keys(this.attrs).length || delete this.attrs;
  return !0;
};

JSAPI.prototype.addAttr = function(attr) {
  if (void 0 === (attr = attr || {}).name || void 0 === attr.prefix || void 0 === attr.local) return !1;
  this.attrs = this.attrs || {};
  this.attrs[attr.name] = attr;
  "class" === attr.name && this.class.hasClass();
  "style" === attr.name && this.style.hasStyle();
  return this.attrs[attr.name];
};

JSAPI.prototype.eachAttr = function(callback, context) {
  if (!this.hasAttr()) return !1;
  for (const name in this.attrs) callback.call(context, this.attrs[name]);
  return !0;
};

JSAPI.prototype.someAttr = function(callback, context) {
  if (!this.hasAttr()) return !1;
  for (const name in this.attrs) if (callback.call(context, this.attrs[name])) return !0;
  return !1;
};

JSAPI.prototype.querySelectorAll = function(selectors) {
  const matchedEls = (0, _cssSelect.selectAll)(selectors, this, cssSelectOpts);
  return matchedEls.length > 0 ? matchedEls : null;
};

JSAPI.prototype.querySelector = function(selectors) {
  return (0, _cssSelect.selectOne)(selectors, this, cssSelectOpts);
};

JSAPI.prototype.matches = function(selector) {
  return (0, _cssSelect.is)(this, selector, cssSelectOpts);
};