"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.SVG2JS = void 0;

var _sax = (obj = require("sax")) && obj.__esModule ? obj : {
  default: obj
};

var obj;

var _jsAPI = require("./jsAPI");

var _cssClassList = require("./css-class-list");

var _cssStyleDeclaration = require("./css-style-declaration");

const entityDeclaration = /<!ENTITY\s+(\S+)\s+(?:'([^']+)'|"([^"]+)")\s*>/g;

const config = {
  strict: !0,
  trim: !1,
  normalize: !0,
  lowercase: !0,
  xmlns: !0,
  position: !0
};

exports.SVG2JS = function(data, callback) {
  const sax = _sax.default.parser(config.strict, config);
  const root = new _jsAPI.JSAPI({
    elem: "#document",
    content: []
  });
  let current = root;
  const stack = [ root ];
  let textContext = null;
  let parsingError = !1;
  function pushToContent(content) {
    content = new _jsAPI.JSAPI(content, current);
    (current.content = current.content || []).push(content);
    return content;
  }
  sax.ondoctype = function(doctype) {
    pushToContent({
      doctype
    });
    const subsetStart = doctype.indexOf("[");
    let entityMatch;
    if (subsetStart >= 0) {
      entityDeclaration.lastIndex = subsetStart;
      for (;null != (entityMatch = entityDeclaration.exec(data)); ) sax.ENTITIES[entityMatch[1]] = entityMatch[2] || entityMatch[3];
    }
  };
  sax.onprocessinginstruction = function(data) {
    pushToContent({
      processinginstruction: data
    });
  };
  sax.oncomment = function(comment) {
    pushToContent({
      comment: comment.trim()
    });
  };
  sax.oncdata = function(cdata) {
    pushToContent({
      cdata
    });
  };
  sax.onopentag = function(data) {
    let elem = {
      elem: data.name,
      prefix: data.prefix,
      local: data.local,
      attrs: {}
    };
    elem.class = new _cssClassList.CSSClassList(elem);
    elem.style = new _cssStyleDeclaration.CSSStyleDeclaration(elem);
    if (Object.keys(data.attributes).length) for (const name in data.attributes) {
      "class" === name && elem.class.hasClass();
      "style" === name && elem.style.hasStyle();
      elem.attrs[name] = {
        name,
        value: data.attributes[name].value,
        prefix: data.attributes[name].prefix,
        local: data.attributes[name].local
      };
    }
    elem = pushToContent(elem);
    current = elem;
    "text" !== data.name || data.prefix || (textContext = current);
    stack.push(elem);
  };
  sax.ontext = function(text) {
    if (/\S/.test(text) || textContext) {
      textContext || (text = text.trim());
      pushToContent({
        text
      });
    }
  };
  sax.onclosetag = function() {
    if (stack.pop() === textContext) {
      !function(elem) {
        if (!elem.content) return elem;
        let start = elem.content[0];
        let end = elem.content[elem.content.length - 1];
        for (;start && start.content && !start.text; ) start = start.content[0];
        start && start.text && (start.text = start.text.replace(/^\s+/, ""));
        for (;end && end.content && !end.text; ) end = end.content[end.content.length - 1];
        end && end.text && (end.text = end.text.replace(/\s+$/, ""));
      }(textContext);
      textContext = null;
    }
    current = stack[stack.length - 1];
  };
  sax.onerror = function(e) {
    e.message = "Error in parsing SVG: " + e.message;
    if (e.message.indexOf("Unexpected end") < 0) throw e;
  };
  sax.onend = function() {
    this.error ? callback({
      error: this.error.message
    }) : callback(root);
  };
  try {
    sax.write(data);
  } catch (e) {
    callback({
      error: e.message
    });
    parsingError = !0;
  }
  parsingError || sax.close();
};