"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.intersects = exports.js2path = exports.computeQuadraticBoundingBox = exports.computeCubicBoundingBox = exports.applyTransforms = exports.relative2absolute = exports.path2js = void 0;

var _tools = require("../lib/svgo/tools");

var _transforms = require("./_transforms");

var _collections = require("./_collections");

const defaultStrokeWidth = _collections.attrsGroupsDefaults.presentation["stroke-width"];

const rNumber = String.raw`[-+]?(?:\d*\.\d+|\d+\.?)(?:[eE][-+]?\d+)?\s*`;

const rCommaWsp = String.raw`(?:\s,?\s*|,\s*)`;

const rNumberCommaWsp = `(${rNumber})` + rCommaWsp;

const rFlagCommaWsp = `([01])${rCommaWsp}?`;

const rCoordinatePair = String.raw`(${rNumber})${rCommaWsp}?(${rNumber})`;

const rArcSeq = (rNumberCommaWsp + "?").repeat(2) + rNumberCommaWsp + rFlagCommaWsp.repeat(2) + rCoordinatePair;

const regPathInstructions = /([MmLlHhVvCcSsQqTtAaZz])\s*/;

const regCoordinateSequence = new RegExp(rNumber, "g");

const regArcArgumentSequence = new RegExp(rArcSeq, "g");

const regNumericValues = /[-+]?(\d*\.\d+|\d+\.?)(?:[eE][-+]?\d+)?/;

exports.path2js = function(path) {
  if (path.pathJS) return path.pathJS;
  const paramsLength = {
    H: 1,
    V: 1,
    M: 2,
    L: 2,
    T: 2,
    Q: 4,
    S: 4,
    C: 6,
    A: 7,
    h: 1,
    v: 1,
    m: 2,
    l: 2,
    t: 2,
    q: 4,
    s: 4,
    c: 6,
    a: 7
  };
  const pathData = [];
  let instruction;
  let startMoveto = !1;
  path.attr("d").value.split(regPathInstructions).forEach((function(data) {
    if (data) {
      if (!startMoveto) {
        if ("M" !== data && "m" !== data) return;
        startMoveto = !0;
      }
      if (regPathInstructions.test(data)) {
        instruction = data;
        "Z" !== instruction && "z" !== instruction || pathData.push({
          instruction: "z"
        });
      } else {
        if ("A" === instruction || "a" === instruction) {
          const newData = [];
          for (let args; args = regArcArgumentSequence.exec(data); ) for (let i = 1; i < args.length; i++) newData.push(args[i]);
          data = newData;
        } else data = data.match(regCoordinateSequence);
        if (!data) return;
        data = data.map(Number);
        if ("M" === instruction || "m" === instruction) {
          pathData.push({
            instruction: 0 === pathData.length ? "M" : instruction,
            data: data.splice(0, 2)
          });
          instruction = "M" === instruction ? "L" : "l";
        }
        for (const pair = paramsLength[instruction]; data.length; ) pathData.push({
          instruction,
          data: data.splice(0, pair)
        });
      }
    }
  }));
  pathData.length && "m" === pathData[0].instruction && (pathData[0].instruction = "M");
  path.pathJS = pathData;
  return pathData;
};

const relative2absolute = function(data) {
  const currentPoint = [ 0, 0 ];
  const subpathPoint = [ 0, 0 ];
  let i;
  return data.map((function(item) {
    const instruction = item.instruction;
    const itemData = item.data && item.data.slice();
    if ("M" === instruction) {
      set(currentPoint, itemData);
      set(subpathPoint, itemData);
    } else if ("mlcsqt".indexOf(instruction) > -1) {
      for (i = 0; i < itemData.length; i++) itemData[i] += currentPoint[i % 2];
      set(currentPoint, itemData);
      "m" === instruction && set(subpathPoint, itemData);
    } else if ("a" === instruction) {
      itemData[5] += currentPoint[0];
      itemData[6] += currentPoint[1];
      set(currentPoint, itemData);
    } else if ("h" === instruction) {
      itemData[0] += currentPoint[0];
      currentPoint[0] = itemData[0];
    } else if ("v" === instruction) {
      itemData[0] += currentPoint[1];
      currentPoint[1] = itemData[0];
    } else "MZLCSQTA".indexOf(instruction) > -1 ? set(currentPoint, itemData) : "H" === instruction ? currentPoint[0] = itemData[0] : "V" === instruction ? currentPoint[1] = itemData[0] : "z" === instruction && set(currentPoint, subpathPoint);
    return "z" === instruction ? {
      instruction: "z"
    } : {
      instruction: instruction.toUpperCase(),
      data: itemData
    };
  }));
};

exports.relative2absolute = relative2absolute;

exports.applyTransforms = function(elem, path, params) {
  if (!elem.hasAttr("transform") || !elem.attr("transform").value || elem.someAttr((function(attr) {
    return ~_collections.referencesProps.indexOf(attr.name) && ~attr.value.indexOf("url(");
  }))) return path;
  const matrix = (0, _transforms.transformsMultiply)((0, _transforms.transform2js)(elem.attr("transform").value));
  const stroke = elem.computedAttr("stroke");
  const id = elem.computedAttr("id");
  const transformPrecision = params.transformPrecision;
  let newPoint;
  let scale;
  if (stroke && "none" !== stroke) {
    if (!params.applyTransformsStroked || (matrix.data[0] !== matrix.data[3] || matrix.data[1] !== -matrix.data[2]) && (matrix.data[0] !== -matrix.data[3] || matrix.data[1] !== matrix.data[2])) return path;
    if (id) {
      let idElem = elem;
      let hasStrokeWidth = !1;
      do {
        idElem.hasAttr("stroke-width") && (hasStrokeWidth = !0);
      } while (!idElem.hasAttr("id", id) && !hasStrokeWidth && (idElem = idElem.parentNode));
      if (!hasStrokeWidth) return path;
    }
    scale = +Math.sqrt(matrix.data[0] * matrix.data[0] + matrix.data[1] * matrix.data[1]).toFixed(transformPrecision);
    if (1 !== scale) {
      const strokeWidth = elem.computedAttr("stroke-width") || defaultStrokeWidth;
      elem.hasAttr("vector-effect") && "non-scaling-stroke" === elem.attr("vector-effect").value || (elem.hasAttr("stroke-width") ? elem.attrs["stroke-width"].value = elem.attrs["stroke-width"].value.trim().replace(regNumericValues, (function(num) {
        return (0, _tools.removeLeadingZero)(num * scale);
      })) : elem.addAttr({
        name: "stroke-width",
        prefix: "",
        local: "stroke-width",
        value: strokeWidth.replace(regNumericValues, (function(num) {
          return (0, _tools.removeLeadingZero)(num * scale);
        }))
      }));
    }
  } else if (id) return path;
  path.forEach((function(pathItem) {
    if (pathItem.data) {
      if ("h" === pathItem.instruction) {
        pathItem.instruction = "l";
        pathItem.data[1] = 0;
      } else if ("v" === pathItem.instruction) {
        pathItem.instruction = "l";
        pathItem.data[1] = pathItem.data[0];
        pathItem.data[0] = 0;
      }
      if ("M" !== pathItem.instruction || 0 === matrix.data[4] && 0 === matrix.data[5]) {
        if ("a" === pathItem.instruction) {
          (0, _transforms.transformArc)(pathItem.data, matrix.data);
          if (Math.abs(pathItem.data[2]) > 80) {
            const a = pathItem.data[0];
            const rotation = pathItem.data[2];
            pathItem.data[0] = pathItem.data[1];
            pathItem.data[1] = a;
            pathItem.data[2] = rotation + (rotation > 0 ? -90 : 90);
          }
          newPoint = transformPoint(matrix.data, pathItem.data[5], pathItem.data[6]);
          pathItem.data[5] = newPoint[0];
          pathItem.data[6] = newPoint[1];
        } else for (let i = 0; i < pathItem.data.length; i += 2) {
          newPoint = transformPoint(matrix.data, pathItem.data[i], pathItem.data[i + 1]);
          pathItem.data[i] = newPoint[0];
          pathItem.data[i + 1] = newPoint[1];
        }
        pathItem.coords[0] = pathItem.base[0] + pathItem.data[pathItem.data.length - 2];
        pathItem.coords[1] = pathItem.base[1] + pathItem.data[pathItem.data.length - 1];
      } else {
        newPoint = transformPoint(matrix.data, pathItem.data[0], pathItem.data[1]);
        set(pathItem.data, newPoint);
        set(pathItem.coords, newPoint);
        matrix.data[4] = 0;
        matrix.data[5] = 0;
      }
    }
  }));
  elem.removeAttr("transform");
  return path;
};

function transformPoint(matrix, x, y) {
  return [ matrix[0] * x + matrix[2] * y + matrix[4], matrix[1] * x + matrix[3] * y + matrix[5] ];
}

exports.computeCubicBoundingBox = function(xa, ya, xb, yb, xc, yc, xd, yd) {
  let minx = Number.POSITIVE_INFINITY;
  let miny = Number.POSITIVE_INFINITY;
  let maxx = Number.NEGATIVE_INFINITY;
  let maxy = Number.NEGATIVE_INFINITY;
  xa < minx && (minx = xa);
  xa > maxx && (maxx = xa);
  xd < minx && (minx = xd);
  xd > maxx && (maxx = xd);
  const tsx = computeCubicFirstDerivativeRoots(xa, xb, xc, xd);
  for (let i = 0; i < tsx.length; i++) {
    const t = tsx[i];
    if (t >= 0 && t <= 1) {
      const x = computeCubicBaseValue(t, xa, xb, xc, xd);
      x < minx && (minx = x);
      x > maxx && (maxx = x);
    }
  }
  ya < miny && (miny = ya);
  ya > maxy && (maxy = ya);
  yd < miny && (miny = yd);
  yd > maxy && (maxy = yd);
  const tsy = computeCubicFirstDerivativeRoots(ya, yb, yc, yd);
  for (let i = 0; i < tsy.length; i++) {
    const t = tsy[i];
    if (t >= 0 && t <= 1) {
      const y = computeCubicBaseValue(t, ya, yb, yc, yd);
      y < miny && (miny = y);
      y > maxy && (maxy = y);
    }
  }
  return {
    minx,
    miny,
    maxx,
    maxy
  };
};

function computeCubicBaseValue(t, a, b, c, d) {
  const mt = 1 - t;
  return mt * mt * mt * a + 3 * mt * mt * t * b + 3 * mt * t * t * c + t * t * t * d;
}

function computeCubicFirstDerivativeRoots(a, b, c, d) {
  const result = [ -1, -1 ];
  const tl = 2 * b - a - c;
  const tr = -Math.sqrt(-a * (c - d) + b * b - b * (c + d) + c * c);
  const dn = 3 * b - a - 3 * c + d;
  if (0 !== dn) {
    result[0] = (tl + tr) / dn;
    result[1] = (tl - tr) / dn;
  }
  return result;
}

exports.computeQuadraticBoundingBox = function(xa, ya, xb, yb, xc, yc) {
  let minx = Number.POSITIVE_INFINITY;
  let miny = Number.POSITIVE_INFINITY;
  let maxx = Number.NEGATIVE_INFINITY;
  let maxy = Number.NEGATIVE_INFINITY;
  xa < minx && (minx = xa);
  xa > maxx && (maxx = xa);
  xc < minx && (minx = xc);
  xc > maxx && (maxx = xc);
  const tx = computeQuadraticFirstDerivativeRoot(xa, xb, xc);
  if (tx >= 0 && tx <= 1) {
    const x = computeQuadraticBaseValue(tx, xa, xb, xc);
    x < minx && (minx = x);
    x > maxx && (maxx = x);
  }
  ya < miny && (miny = ya);
  ya > maxy && (maxy = ya);
  yc < miny && (miny = yc);
  yc > maxy && (maxy = yc);
  const ty = computeQuadraticFirstDerivativeRoot(ya, yb, yc);
  if (ty >= 0 && ty <= 1) {
    const y = computeQuadraticBaseValue(ty, ya, yb, yc);
    y < miny && (miny = y);
    y > maxy && (maxy = y);
  }
  return {
    minx,
    miny,
    maxx,
    maxy
  };
};

function computeQuadraticBaseValue(t, a, b, c) {
  const mt = 1 - t;
  return mt * mt * a + 2 * mt * t * b + t * t * c;
}

function computeQuadraticFirstDerivativeRoot(a, b, c) {
  let t = -1;
  const denominator = a - 2 * b + c;
  0 !== denominator && (t = (a - b) / denominator);
  return t;
}

exports.js2path = function(path, data, params) {
  path.pathJS = data;
  params.collapseRepeated && (data = function(data) {
    let prev;
    let prevIndex;
    return data.reduce((function(newPath, item) {
      if (prev && item.data && item.instruction === prev.instruction) if ("M" !== item.instruction) prev = newPath[prevIndex] = {
        instruction: prev.instruction,
        data: prev.data.concat(item.data),
        coords: item.coords,
        base: prev.base
      }; else {
        prev.data = item.data;
        prev.coords = item.coords;
      } else {
        newPath.push(item);
        prev = item;
        prevIndex = newPath.length - 1;
      }
      return newPath;
    }), []);
  }(data));
  path.attr("d").value = data.reduce((function(pathString, item) {
    let strData = "";
    item.data && (strData = (0, _tools.cleanupOutData)(item.data, params, item.instruction));
    return pathString + (item.instruction + strData);
  }), "");
};

function set(dest, source) {
  dest[0] = source[source.length - 2];
  dest[1] = source[source.length - 1];
  return dest;
}

exports.intersects = function(path1, path2) {
  if (path1.length < 3 || path2.length < 3) return !1;
  const points1 = relative2absolute(path1).reduce(gatherPoints, []);
  const points2 = relative2absolute(path2).reduce(gatherPoints, []);
  if (points1.maxX <= points2.minX || points2.maxX <= points1.minX || points1.maxY <= points2.minY || points2.maxY <= points1.minY || points1.every((function(set1) {
    return points2.every((function(set2) {
      return set1[set1.maxX][0] <= set2[set2.minX][0] || set2[set2.maxX][0] <= set1[set1.minX][0] || set1[set1.maxY][1] <= set2[set2.minY][1] || set2[set2.maxY][1] <= set1[set1.minY][1];
    }));
  }))) return !1;
  const hullNest1 = points1.map(convexHull);
  const hullNest2 = points2.map(convexHull);
  return hullNest1.some((function(hull1) {
    return !(hull1.length < 3) && hullNest2.some((function(hull2) {
      if (hull2.length < 3) return !1;
      const simplex = [ getSupport(hull1, hull2, [ 1, 0 ]) ];
      const direction = minus(simplex[0]);
      let iterations = 1e4;
      for (;;) {
        if (0 == iterations--) {
          console.error("Error: infinite loop while processing mergePaths plugin.");
          return !0;
        }
        simplex.push(getSupport(hull1, hull2, direction));
        if (dot(direction, simplex[simplex.length - 1]) <= 0) return !1;
        if (processSimplex(simplex, direction)) return !0;
      }
    }));
  }));
  function getSupport(a, b, direction) {
    return sub(supportPoint(a, direction), supportPoint(b, minus(direction)));
  }
  function supportPoint(polygon, direction) {
    let index = direction[1] >= 0 ? direction[0] < 0 ? polygon.maxY : polygon.maxX : direction[0] < 0 ? polygon.minX : polygon.minY;
    let max = -1 / 0;
    let value;
    for (;(value = dot(polygon[index], direction)) > max; ) {
      max = value;
      index = ++index % polygon.length;
    }
    return polygon[(index || polygon.length) - 1];
  }
};

function processSimplex(simplex, direction) {
  if (2 === simplex.length) {
    const a = simplex[1];
    const b = simplex[0];
    const AO = minus(simplex[1]);
    const AB = sub(b, a);
    if (dot(AO, AB) > 0) set(direction, orth(AB, a)); else {
      set(direction, AO);
      simplex.shift();
    }
  } else {
    const a = simplex[2];
    const b = simplex[1];
    const c = simplex[0];
    const AB = sub(b, a);
    const AC = sub(c, a);
    const AO = minus(a);
    const ACB = orth(AB, AC);
    const ABC = orth(AC, AB);
    if (dot(ACB, AO) > 0) if (dot(AB, AO) > 0) {
      set(direction, ACB);
      simplex.shift();
    } else {
      set(direction, AO);
      simplex.splice(0, 2);
    } else {
      if (!(dot(ABC, AO) > 0)) return !0;
      if (dot(AC, AO) > 0) {
        set(direction, ABC);
        simplex.splice(1, 1);
      } else {
        set(direction, AO);
        simplex.splice(0, 2);
      }
    }
  }
  return !1;
}

function minus(v) {
  return [ -v[0], -v[1] ];
}

function sub(v1, v2) {
  return [ v1[0] - v2[0], v1[1] - v2[1] ];
}

function dot(v1, v2) {
  return v1[0] * v2[0] + v1[1] * v2[1];
}

function orth(v, from) {
  const o = [ -v[1], v[0] ];
  return dot(o, minus(from)) < 0 ? minus(o) : o;
}

let prevCtrlPoint;

function gatherPoints(points, item, index, path) {
  let subPath = points.length && points[points.length - 1];
  const prev = index && path[index - 1];
  let basePoint = subPath.length && subPath[subPath.length - 1];
  const data = item.data;
  let ctrlPoint = basePoint;
  switch (item.instruction) {
   case "M":
    points.push(subPath = []);
    break;

   case "H":
    addPoint(subPath, [ data[0], basePoint[1] ]);
    break;

   case "V":
    addPoint(subPath, [ basePoint[0], data[0] ]);
    break;

   case "Q":
    addPoint(subPath, data.slice(0, 2));
    prevCtrlPoint = [ data[2] - data[0], data[3] - data[1] ];
    break;

   case "T":
    if ("Q" === prev.instruction || "T" === prev.instruction) {
      ctrlPoint = [ basePoint[0] + prevCtrlPoint[0], basePoint[1] + prevCtrlPoint[1] ];
      addPoint(subPath, ctrlPoint);
      prevCtrlPoint = [ data[0] - ctrlPoint[0], data[1] - ctrlPoint[1] ];
    }
    break;

   case "C":
    addPoint(subPath, [ .5 * (basePoint[0] + data[0]), .5 * (basePoint[1] + data[1]) ]);
    addPoint(subPath, [ .5 * (data[0] + data[2]), .5 * (data[1] + data[3]) ]);
    addPoint(subPath, [ .5 * (data[2] + data[4]), .5 * (data[3] + data[5]) ]);
    prevCtrlPoint = [ data[4] - data[2], data[5] - data[3] ];
    break;

   case "S":
    if ("C" === prev.instruction || "S" === prev.instruction) {
      addPoint(subPath, [ basePoint[0] + .5 * prevCtrlPoint[0], basePoint[1] + .5 * prevCtrlPoint[1] ]);
      ctrlPoint = [ basePoint[0] + prevCtrlPoint[0], basePoint[1] + prevCtrlPoint[1] ];
    }
    addPoint(subPath, [ .5 * (ctrlPoint[0] + data[0]), .5 * (ctrlPoint[1] + data[1]) ]);
    addPoint(subPath, [ .5 * (data[0] + data[2]), .5 * (data[1] + data[3]) ]);
    prevCtrlPoint = [ data[2] - data[0], data[3] - data[1] ];
    break;

   case "A":
    {
      const curves = a2c.apply(0, basePoint.concat(data));
      for (let cData; (cData = curves.splice(0, 6).map(toAbsolute)).length; ) {
        addPoint(subPath, [ .5 * (basePoint[0] + cData[0]), .5 * (basePoint[1] + cData[1]) ]);
        addPoint(subPath, [ .5 * (cData[0] + cData[2]), .5 * (cData[1] + cData[3]) ]);
        addPoint(subPath, [ .5 * (cData[2] + cData[4]), .5 * (cData[3] + cData[5]) ]);
        curves.length && addPoint(subPath, basePoint = cData.slice(-2));
      }
      break;
    }
  }
  data && data.length >= 2 && addPoint(subPath, data.slice(-2));
  return points;
  function toAbsolute(n, i) {
    return n + basePoint[i % 2];
  }
  function addPoint(path, point) {
    if (!path.length || point[1] > path[path.maxY][1]) {
      path.maxY = path.length;
      points.maxY = points.length ? Math.max(point[1], points.maxY) : point[1];
    }
    if (!path.length || point[0] > path[path.maxX][0]) {
      path.maxX = path.length;
      points.maxX = points.length ? Math.max(point[0], points.maxX) : point[0];
    }
    if (!path.length || point[1] < path[path.minY][1]) {
      path.minY = path.length;
      points.minY = points.length ? Math.min(point[1], points.minY) : point[1];
    }
    if (!path.length || point[0] < path[path.minX][0]) {
      path.minX = path.length;
      points.minX = points.length ? Math.min(point[0], points.minX) : point[0];
    }
    path.push(point);
  }
}

function convexHull(points) {
  points.sort((function(a, b) {
    return a[0] === b[0] ? a[1] - b[1] : a[0] - b[0];
  }));
  const lower = [];
  let minY = 0;
  let bottom = 0;
  for (let i = 0; i < points.length; i++) {
    for (;lower.length >= 2 && cross(lower[lower.length - 2], lower[lower.length - 1], points[i]) <= 0; ) lower.pop();
    if (points[i][1] < points[minY][1]) {
      minY = i;
      bottom = lower.length;
    }
    lower.push(points[i]);
  }
  const upper = [];
  let maxY = points.length - 1;
  let top = 0;
  for (let i = points.length; i--; ) {
    for (;upper.length >= 2 && cross(upper[upper.length - 2], upper[upper.length - 1], points[i]) <= 0; ) upper.pop();
    if (points[i][1] > points[maxY][1]) {
      maxY = i;
      top = upper.length;
    }
    upper.push(points[i]);
  }
  upper.pop();
  lower.pop();
  const hull = lower.concat(upper);
  hull.minX = 0;
  hull.maxX = lower.length;
  hull.minY = bottom;
  hull.maxY = (lower.length + top) % hull.length;
  return hull;
}

function cross(o, a, b) {
  return (a[0] - o[0]) * (b[1] - o[1]) - (a[1] - o[1]) * (b[0] - o[0]);
}

function a2c(x1, y1, rx, ry, angle, largeArcFlag, sweepFlag, x2, y2, recursive) {
  const _120 = 120 * Math.PI / 180;
  const rad = Math.PI / 180 * (+angle || 0);
  let res = [];
  let cy;
  let cx;
  let f1;
  let f2;
  const rotateX = function(x, y, rad) {
    return x * Math.cos(rad) - y * Math.sin(rad);
  };
  const rotateY = function(x, y, rad) {
    return x * Math.sin(rad) + y * Math.cos(rad);
  };
  if (recursive) {
    f1 = recursive[0];
    f2 = recursive[1];
    cx = recursive[2];
    cy = recursive[3];
  } else {
    y1 = rotateY(x1 = rotateX(x1, y1, -rad), y1, -rad);
    const x = (x1 - (x2 = rotateX(x2, y2, -rad))) / 2;
    const y = (y1 - (y2 = rotateY(x2, y2, -rad))) / 2;
    let h = x * x / (rx * rx) + y * y / (ry * ry);
    if (h > 1) {
      h = Math.sqrt(h);
      rx *= h;
      ry *= h;
    }
    const rx2 = rx * rx;
    const ry2 = ry * ry;
    const k = (largeArcFlag === sweepFlag ? -1 : 1) * Math.sqrt(Math.abs((rx2 * ry2 - rx2 * y * y - ry2 * x * x) / (rx2 * y * y + ry2 * x * x)));
    cx = k * rx * y / ry + (x1 + x2) / 2;
    cy = k * -ry * x / rx + (y1 + y2) / 2;
    f1 = Math.asin(((y1 - cy) / ry).toFixed(9));
    f2 = Math.asin(((y2 - cy) / ry).toFixed(9));
    f1 = x1 < cx ? Math.PI - f1 : f1;
    f2 = x2 < cx ? Math.PI - f2 : f2;
    f1 < 0 && (f1 = 2 * Math.PI + f1);
    f2 < 0 && (f2 = 2 * Math.PI + f2);
    sweepFlag && f1 > f2 && (f1 -= 2 * Math.PI);
    !sweepFlag && f2 > f1 && (f2 -= 2 * Math.PI);
  }
  let df = f2 - f1;
  if (Math.abs(df) > _120) {
    const f2old = f2;
    const x2old = x2;
    const y2old = y2;
    f2 = f1 + _120 * (sweepFlag && f2 > f1 ? 1 : -1);
    res = a2c(x2 = cx + rx * Math.cos(f2), y2 = cy + ry * Math.sin(f2), rx, ry, angle, 0, sweepFlag, x2old, y2old, [ f2, f2old, cx, cy ]);
  }
  df = f2 - f1;
  const c1 = Math.cos(f1);
  const s1 = Math.sin(f1);
  const c2 = Math.cos(f2);
  const s2 = Math.sin(f2);
  const t = Math.tan(df / 4);
  const hx = 4 / 3 * rx * t;
  const hy = 4 / 3 * ry * t;
  const m = [ -hx * s1, hy * c1, x2 + hx * s2 - x1, y2 - hy * c2 - y1, x2 - x1, y2 - y1 ];
  if (recursive) return m.concat(res);
  {
    res = m.concat(res);
    const newres = [];
    for (let i = 0, n = res.length; i < n; i++) newres[i] = i % 2 ? rotateY(res[i - 1], res[i], rad) : rotateX(res[i], res[i + 1], rad);
    return newres;
  }
}