"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.transformArc = exports.matrixToTransform = exports.mth = exports.transformsMultiply = exports.transform2js = void 0;

const regTransformTypes = /matrix|translate|scale|rotate|skewX|skewY/;

const regTransformSplit = /\s*(matrix|translate|scale|rotate|skewX|skewY)\s*\(\s*(.+?)\s*\)[\s,]*/;

const regNumericValues = /[-+]?(?:\d*\.\d+|\d+\.?)(?:[eE][-+]?\d+)?/g;

exports.transform2js = function(transformString) {
  const transforms = [];
  let current;
  transformString.split(regTransformSplit).forEach((function(item) {
    let num;
    if (item) if (regTransformTypes.test(item)) transforms.push(current = {
      name: item
    }); else for (;num = regNumericValues.exec(item); ) {
      num = Number(num);
      current.data ? current.data.push(num) : current.data = [ num ];
    }
  }));
  return current && current.data ? transforms : [];
};

exports.transformsMultiply = function(transforms) {
  return {
    name: "matrix",
    data: (transforms = transforms.map((function(transform) {
      return "matrix" === transform.name ? transform.data : function(transform) {
        if ("matrix" === transform.name) return transform.data;
        let matrix;
        switch (transform.name) {
         case "translate":
          matrix = [ 1, 0, 0, 1, transform.data[0], transform.data[1] || 0 ];
          break;

         case "scale":
          matrix = [ transform.data[0], 0, 0, transform.data[1] || transform.data[0], 0, 0 ];
          break;

         case "rotate":
          {
            const cos = mth.cos(transform.data[0]);
            const sin = mth.sin(transform.data[0]);
            const cx = transform.data[1] || 0;
            const cy = transform.data[2] || 0;
            matrix = [ cos, sin, -sin, cos, (1 - cos) * cx + sin * cy, (1 - cos) * cy - sin * cx ];
            break;
          }

         case "skewX":
          matrix = [ 1, 0, mth.tan(transform.data[0]), 1, 0, 0 ];
          break;

         case "skewY":
          matrix = [ 1, mth.tan(transform.data[0]), 0, 1, 0, 0 ];
        }
        return matrix;
      }(transform);
    }))).length > 0 ? transforms.reduce(multiplyTransformMatrices) : []
  };
};

const mth = {
  rad: function(deg) {
    return deg * Math.PI / 180;
  },
  deg: function(rad) {
    return 180 * rad / Math.PI;
  },
  cos: function(deg) {
    return Math.cos(this.rad(deg));
  },
  acos: function(val, floatPrecision) {
    return +this.deg(Math.acos(val)).toFixed(floatPrecision);
  },
  sin: function(deg) {
    return Math.sin(this.rad(deg));
  },
  asin: function(val, floatPrecision) {
    return +this.deg(Math.asin(val)).toFixed(floatPrecision);
  },
  tan: function(deg) {
    return Math.tan(this.rad(deg));
  },
  atan: function(val, floatPrecision) {
    return +this.deg(Math.atan(val)).toFixed(floatPrecision);
  }
};

exports.mth = mth;

exports.matrixToTransform = function(transform, params) {
  const floatPrecision = params.floatPrecision;
  const data = transform.data;
  const transforms = [];
  let sx = +Math.hypot(data[0], data[1]).toFixed(params.transformPrecision);
  let sy = +((data[0] * data[3] - data[1] * data[2]) / sx).toFixed(params.transformPrecision);
  const colsSum = data[0] * data[2] + data[1] * data[3];
  const rowsSum = data[0] * data[1] + data[2] * data[3];
  const scaleBefore = 0 !== rowsSum || sx === sy;
  (data[4] || data[5]) && transforms.push({
    name: "translate",
    data: data.slice(4, data[5] ? 6 : 5)
  });
  if (!data[1] && data[2]) transforms.push({
    name: "skewX",
    data: [ mth.atan(data[2] / sy, floatPrecision) ]
  }); else if (data[1] && !data[2]) {
    transforms.push({
      name: "skewY",
      data: [ mth.atan(data[1] / data[0], floatPrecision) ]
    });
    sx = data[0];
    sy = data[3];
  } else if (!colsSum || 1 === sx && 1 === sy || !scaleBefore) {
    if (!scaleBefore) {
      sx = (data[0] < 0 ? -1 : 1) * Math.hypot(data[0], data[2]);
      sy = (data[3] < 0 ? -1 : 1) * Math.hypot(data[1], data[3]);
      transforms.push({
        name: "scale",
        data: [ sx, sy ]
      });
    }
    const angle = Math.min(Math.max(-1, data[0] / sx), 1);
    const rotate = [ mth.acos(angle, floatPrecision) * ((scaleBefore ? 1 : sy) * data[1] < 0 ? -1 : 1) ];
    rotate[0] && transforms.push({
      name: "rotate",
      data: rotate
    });
    rowsSum && colsSum && transforms.push({
      name: "skewX",
      data: [ mth.atan(colsSum / (sx * sx), floatPrecision) ]
    });
    if (rotate[0] && (data[4] || data[5])) {
      transforms.shift();
      const cos = data[0] / sx;
      const sin = data[1] / (scaleBefore ? sx : sy);
      const x = data[4] * (scaleBefore || sy);
      const y = data[5] * (scaleBefore || sx);
      const denom = (Math.pow(1 - cos, 2) + Math.pow(sin, 2)) * (scaleBefore || sx * sy);
      rotate.push(((1 - cos) * x - sin * y) / denom);
      rotate.push(((1 - cos) * y + sin * x) / denom);
    }
  } else if (data[1] || data[2]) return transform;
  (!scaleBefore || 1 === sx && 1 === sy) && transforms.length || transforms.push({
    name: "scale",
    data: sx === sy ? [ sx ] : [ sx, sy ]
  });
  return transforms;
};

exports.transformArc = function(arc, transform) {
  let a = arc[0];
  let b = arc[1];
  const rot = arc[2] * Math.PI / 180;
  const cos = Math.cos(rot);
  const sin = Math.sin(rot);
  let h = Math.pow(arc[5] * cos + arc[6] * sin, 2) / (4 * a * a) + Math.pow(arc[6] * cos - arc[5] * sin, 2) / (4 * b * b);
  if (h > 1) {
    h = Math.sqrt(h);
    a *= h;
    b *= h;
  }
  const m = multiplyTransformMatrices(transform, [ a * cos, a * sin, -b * sin, b * cos, 0, 0 ]);
  const lastCol = m[2] * m[2] + m[3] * m[3];
  const squareSum = m[0] * m[0] + m[1] * m[1] + lastCol;
  const root = Math.hypot(m[0] - m[3], m[1] + m[2]) * Math.hypot(m[0] + m[3], m[1] - m[2]);
  if (root) {
    const majorAxisSqr = (squareSum + root) / 2;
    const minorAxisSqr = (squareSum - root) / 2;
    const major = Math.abs(majorAxisSqr - lastCol) > 1e-6;
    const sub = (major ? majorAxisSqr : minorAxisSqr) - lastCol;
    const rowsSum = m[0] * m[2] + m[1] * m[3];
    const term1 = m[0] * sub + m[2] * rowsSum;
    const term2 = m[1] * sub + m[3] * rowsSum;
    arc[0] = Math.sqrt(majorAxisSqr);
    arc[1] = Math.sqrt(minorAxisSqr);
    arc[2] = ((major ? term2 < 0 : term1 > 0) ? -1 : 1) * Math.acos((major ? term1 : term2) / Math.hypot(term1, term2)) * 180 / Math.PI;
  } else {
    arc[0] = arc[1] = Math.sqrt(squareSum / 2);
    arc[2] = 0;
  }
  transform[0] < 0 != transform[3] < 0 && (arc[4] = 1 - arc[4]);
  return arc;
};

function multiplyTransformMatrices(a, b) {
  return [ a[0] * b[0] + a[2] * b[1], a[1] * b[0] + a[3] * b[1], a[0] * b[2] + a[2] * b[3], a[1] * b[2] + a[3] * b[3], a[0] * b[4] + a[2] * b[5] + a[4], a[1] * b[4] + a[3] * b[5] + a[5] ];
}