"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.fn = exports.params = exports.description = exports.active = exports.type = exports.name = void 0;

var _collections = require("./_collections");

const referencesProps = new Set(_collections.referencesProps);

exports.name = "cleanupIDs";

exports.type = "full";

exports.active = !0;

exports.description = "removes unused IDs and minifies used";

exports.params = {
  remove: !0,
  minify: !0,
  prefix: "",
  preserve: [],
  preservePrefixes: [],
  force: !1
};

const regReferencesUrl = /\burl\(("|')?#(.+?)\1\)/;

const regReferencesHref = /^#(.+?)$/;

const regReferencesBegin = /(\w+)\./;

const styleOrScript = [ "style", "script" ];

const generateIDchars = [ "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z" ];

const maxIDindex = generateIDchars.length - 1;

exports.fn = function(data, params) {
  let currentID;
  let currentIDstring;
  const IDs = new Map;
  const referencesIDs = new Map;
  let hasStyleOrScript = !1;
  const preserveIDs = new Set(Array.isArray(params.preserve) ? params.preserve : params.preserve ? [ params.preserve ] : []);
  const preserveIDPrefixes = new Set(Array.isArray(params.preservePrefixes) ? params.preservePrefixes : params.preservePrefixes ? [ params.preservePrefixes ] : []);
  data = function monkeys(items) {
    for (let i = 0; i < items.content.length && !hasStyleOrScript; i++) {
      const item = items.content[i];
      if (!params.force) {
        const isNotEmpty = Boolean(item.content);
        if (item.isElem(styleOrScript) && isNotEmpty) {
          hasStyleOrScript = !0;
          continue;
        }
        if (item.isElem("svg")) {
          let hasDefsOnly = !0;
          for (let j = 0; j < item.content.length; j++) if (!item.content[j].isElem("defs")) {
            hasDefsOnly = !1;
            break;
          }
          if (hasDefsOnly) break;
        }
      }
      item.isElem() && item.eachAttr((function(attr) {
        let key;
        let match;
        if ("id" !== attr.name) {
          referencesProps.has(attr.name) && (match = attr.value.match(regReferencesUrl)) ? key = match[2] : ("href" === attr.local && (match = attr.value.match(regReferencesHref)) || "begin" === attr.name && (match = attr.value.match(regReferencesBegin))) && (key = match[1]);
          if (key) {
            const ref = referencesIDs.get(key) || [];
            ref.push(attr);
            referencesIDs.set(key, ref);
          }
        } else {
          key = attr.value;
          IDs.has(key) ? item.removeAttr("id") : IDs.set(key, item);
        }
      }));
      item.content && monkeys(item);
    }
    return items;
  }(data);
  if (hasStyleOrScript) return data;
  const idPreserved = id => preserveIDs.has(id) || function(prefixArray, currentID) {
    if (!currentID) return !1;
    for (const prefix of prefixArray) if (currentID.startsWith(prefix)) return !0;
    return !1;
  }(preserveIDPrefixes, id);
  for (const ref of referencesIDs) {
    const key = ref[0];
    if (IDs.has(key)) {
      if (params.minify && !idPreserved(key)) {
        do {
          currentIDstring = getIDstring(currentID = generateID(currentID), params);
        } while (idPreserved(currentIDstring));
        IDs.get(key).attr("id").value = currentIDstring;
        for (const attr of ref[1]) attr.value = attr.value.includes("#") ? attr.value.replace("#" + key, "#" + currentIDstring) : attr.value.replace(key + ".", currentIDstring + ".");
      }
      IDs.delete(key);
    }
  }
  if (params.remove) for (const keyElem of IDs) idPreserved(keyElem[0]) || keyElem[1].removeAttr("id");
  return data;
};

function generateID(currentID) {
  if (!currentID) return [ 0 ];
  currentID[currentID.length - 1]++;
  for (let i = currentID.length - 1; i > 0; i--) if (currentID[i] > maxIDindex) {
    currentID[i] = 0;
    void 0 !== currentID[i - 1] && currentID[i - 1]++;
  }
  if (currentID[0] > maxIDindex) {
    currentID[0] = 0;
    currentID.unshift(0);
  }
  return currentID;
}

function getIDstring(arr, params) {
  return params.prefix + arr.map((i => generateIDchars[i])).join("");
}