"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.fn = exports.params = exports.description = exports.active = exports.type = exports.name = void 0;

var _tools = require("../lib/svgo/tools");

exports.name = "cleanupNumericValues";

exports.type = "perItem";

exports.active = !0;

exports.description = "rounds numeric values to the fixed precision, removes default ‘px’ units";

exports.params = {
  floatPrecision: 3,
  leadingZero: !0,
  defaultPx: !0,
  convertToPx: !0
};

const regNumericValues = /^([-+]?\d*\.?\d+([eE][-+]?\d+)?)(px|pt|pc|mm|cm|m|in|ft|em|ex|%)?$/;

const absoluteLengths = {
  cm: 96 / 2.54,
  mm: 96 / 25.4,
  in: 96,
  pt: 4 / 3,
  pc: 16
};

exports.fn = function(item, params) {
  if (item.isElem()) {
    const floatPrecision = params.floatPrecision;
    if (item.hasAttr("viewBox")) {
      const nums = item.attr("viewBox").value.split(/\s,?\s*|,\s*/g);
      item.attr("viewBox").value = nums.map((function(value) {
        const num = +value;
        return isNaN(num) ? value : +num.toFixed(floatPrecision);
      })).join(" ");
    }
    item.eachAttr((function(attr) {
      if ("version" === attr.name) return;
      const match = attr.value.match(regNumericValues);
      if (match) {
        let num = +(+match[1]).toFixed(floatPrecision);
        let units = match[3] || "";
        if (params.convertToPx && units && units in absoluteLengths) {
          const pxNum = +(absoluteLengths[units] * match[1]).toFixed(floatPrecision);
          if (String(pxNum).length < match[0].length) {
            num = pxNum;
            units = "px";
          }
        }
        params.leadingZero && (num = (0, _tools.removeLeadingZero)(num));
        params.defaultPx && "px" === units && (units = "");
        attr.value = num + units;
      }
    }));
  }
};