"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.fn = exports.params = exports.description = exports.active = exports.type = exports.name = void 0;

var _tools = require("../lib/svgo/tools");

var _collections = require("./_collections");

var _path = require("./_path");

exports.name = "convertPathData";

exports.type = "perItem";

exports.active = !0;

exports.description = "optimizes path data: writes in shorter form, applies transformations";

exports.params = {
  applyTransforms: !0,
  applyTransformsStroked: !0,
  makeArcs: {
    threshold: 2.5,
    tolerance: .5
  },
  straightCurves: !0,
  lineShorthands: !0,
  curveSmoothShorthands: !0,
  floatPrecision: 3,
  transformPrecision: 5,
  removeUseless: !0,
  collapseRepeated: !0,
  utilizeAbsolute: !0,
  leadingZero: !0,
  negativeExtraSpace: !0,
  noSpaceAfterFlags: !0,
  forceAbsolutePath: !1
};

let roundData;

let precision;

let error;

let arcThreshold;

let arcTolerance;

let hasMarkerMid;

let hasStrokeLinecap;

exports.fn = function(item, params) {
  if (item.isElem(_collections.pathElems) && item.hasAttr("d")) {
    precision = params.floatPrecision;
    error = !1 !== precision ? +Math.pow(.1, precision).toFixed(precision) : .01;
    roundData = precision > 0 && precision < 20 ? strongRound : round;
    if (params.makeArcs) {
      arcThreshold = params.makeArcs.threshold;
      arcTolerance = params.makeArcs.tolerance;
    }
    hasMarkerMid = item.hasAttr("marker-mid");
    const stroke = item.computedAttr("stroke");
    const strokeLinecap = item.computedAttr("stroke");
    hasStrokeLinecap = stroke && "none" !== stroke && strokeLinecap && "butt" !== strokeLinecap;
    let data = (0, _path.path2js)(item);
    if (data.length) {
      !function(path) {
        const point = [ 0, 0 ];
        const subpathPoint = [ 0, 0 ];
        let baseItem;
        path.forEach((function(item, index) {
          let instruction = item.instruction;
          const data = item.data;
          if (data) {
            if ("mcslqta".indexOf(instruction) > -1) {
              point[0] += data[data.length - 2];
              point[1] += data[data.length - 1];
              if ("m" === instruction) {
                subpathPoint[0] = point[0];
                subpathPoint[1] = point[1];
                baseItem = item;
              }
            } else "h" === instruction ? point[0] += data[0] : "v" === instruction && (point[1] += data[0]);
            if ("M" === instruction) {
              index > 0 && (instruction = "m");
              data[0] -= point[0];
              data[1] -= point[1];
              subpathPoint[0] = point[0] += data[0];
              subpathPoint[1] = point[1] += data[1];
              baseItem = item;
            } else if ("LT".indexOf(instruction) > -1) {
              instruction = instruction.toLowerCase();
              data[0] -= point[0];
              data[1] -= point[1];
              point[0] += data[0];
              point[1] += data[1];
            } else if ("C" === instruction) {
              instruction = "c";
              data[0] -= point[0];
              data[1] -= point[1];
              data[2] -= point[0];
              data[3] -= point[1];
              data[4] -= point[0];
              data[5] -= point[1];
              point[0] += data[4];
              point[1] += data[5];
            } else if ("SQ".indexOf(instruction) > -1) {
              instruction = instruction.toLowerCase();
              data[0] -= point[0];
              data[1] -= point[1];
              data[2] -= point[0];
              data[3] -= point[1];
              point[0] += data[2];
              point[1] += data[3];
            } else if ("A" === instruction) {
              instruction = "a";
              data[5] -= point[0];
              data[6] -= point[1];
              point[0] += data[5];
              point[1] += data[6];
            } else if ("H" === instruction) {
              instruction = "h";
              data[0] -= point[0];
              point[0] += data[0];
            } else if ("V" === instruction) {
              instruction = "v";
              data[0] -= point[1];
              point[1] += data[0];
            }
            item.instruction = instruction;
            item.data = data;
            item.coords = point.slice(-2);
          } else if ("z" === instruction) {
            baseItem && (item.coords = baseItem.coords);
            point[0] = subpathPoint[0];
            point[1] = subpathPoint[1];
          }
          item.base = index > 0 ? path[index - 1].coords : [ 0, 0 ];
        }));
      }(data);
      params.applyTransforms && (data = (0, _path.applyTransforms)(item, data, params));
      data = function(path, params) {
        const stringify = data2Path.bind(null, params);
        const relSubpoint = [ 0, 0 ];
        const pathBase = [ 0, 0 ];
        let prev = {};
        return path.filter((function(item, index, path) {
          let instruction = item.instruction;
          let data = item.data;
          let next = path[index + 1];
          if (data) {
            let sdata = data;
            let circle;
            if ("s" === instruction) {
              sdata = [ 0, 0 ].concat(data);
              if ("cs".indexOf(prev.instruction) > -1) {
                const pdata = prev.data;
                const n = pdata.length;
                sdata[0] = pdata[n - 2] - pdata[n - 4];
                sdata[1] = pdata[n - 1] - pdata[n - 3];
              }
            }
            if (params.makeArcs && ("c" === instruction || "s" === instruction) && isConvex(sdata) && (circle = function(curve) {
              const midPoint = getCubicBezierPoint(curve, .5);
              const m1 = [ midPoint[0] / 2, midPoint[1] / 2 ];
              const m2 = [ (midPoint[0] + curve[4]) / 2, (midPoint[1] + curve[5]) / 2 ];
              const center = getIntersection([ m1[0], m1[1], m1[0] + m1[1], m1[1] - m1[0], m2[0], m2[1], m2[0] + (m2[1] - midPoint[1]), m2[1] - (m2[0] - midPoint[0]) ]);
              const radius = center && getDistance([ 0, 0 ], center);
              const tolerance = Math.min(arcThreshold * error, arcTolerance * radius / 100);
              if (center && radius < 1e15 && [ 1 / 4, 3 / 4 ].every((function(point) {
                return Math.abs(getDistance(getCubicBezierPoint(curve, point), center) - radius) <= tolerance;
              }))) return {
                center,
                radius
              };
            }(sdata))) {
              let j;
              const r = roundData([ circle.radius ])[0];
              let angle = findArcAngle(sdata, circle);
              const sweep = sdata[5] * sdata[0] - sdata[4] * sdata[1] > 0 ? 1 : 0;
              let arc = {
                instruction: "a",
                data: [ r, r, 0, 0, sweep, sdata[4], sdata[5] ],
                coords: item.coords.slice(),
                base: item.base
              };
              const output = [ arc ];
              const relCenter = [ circle.center[0] - sdata[4], circle.center[1] - sdata[5] ];
              const relCircle = {
                center: relCenter,
                radius: circle.radius
              };
              const arcCurves = [ item ];
              let hasPrev = 0;
              let suffix = "";
              let nextLonghand;
              if ("c" === prev.instruction && isConvex(prev.data) && isArcPrev(prev.data, circle) || "a" === prev.instruction && prev.sdata && isArcPrev(prev.sdata, circle)) {
                arcCurves.unshift(prev);
                arc.base = prev.base;
                arc.data[5] = arc.coords[0] - arc.base[0];
                arc.data[6] = arc.coords[1] - arc.base[1];
                const prevData = "a" === prev.instruction ? prev.sdata : prev.data;
                angle += findArcAngle(prevData, {
                  center: [ prevData[4] + circle.center[0], prevData[5] + circle.center[1] ],
                  radius: circle.radius
                });
                angle > Math.PI && (arc.data[3] = 1);
                hasPrev = 1;
              }
              for (j = index; (next = path[++j]) && ~"cs".indexOf(next.instruction); ) {
                let nextData = next.data;
                if ("s" === next.instruction) {
                  nextLonghand = makeLonghand({
                    instruction: "s",
                    data: next.data.slice()
                  }, path[j - 1].data);
                  nextData = nextLonghand.data;
                  nextLonghand.data = nextData.slice(0, 2);
                  suffix = stringify([ nextLonghand ]);
                }
                if (!isConvex(nextData) || !isArc(nextData, relCircle)) break;
                angle += findArcAngle(nextData, relCircle);
                if (angle - 2 * Math.PI > .001) break;
                angle > Math.PI && (arc.data[3] = 1);
                arcCurves.push(next);
                if (!(2 * Math.PI - angle > .001)) {
                  arc.data[5] = 2 * (relCircle.center[0] - nextData[4]);
                  arc.data[6] = 2 * (relCircle.center[1] - nextData[5]);
                  arc.coords = [ arc.base[0] + arc.data[5], arc.base[1] + arc.data[6] ];
                  arc = {
                    instruction: "a",
                    data: [ r, r, 0, 0, sweep, next.coords[0] - arc.coords[0], next.coords[1] - arc.coords[1] ],
                    coords: next.coords,
                    base: arc.coords
                  };
                  output.push(arc);
                  j++;
                  break;
                }
                arc.coords = next.coords;
                arc.data[5] = arc.coords[0] - arc.base[0];
                arc.data[6] = arc.coords[1] - arc.base[1];
                relCenter[0] -= nextData[4];
                relCenter[1] -= nextData[5];
              }
              if ((stringify(output) + suffix).length < stringify(arcCurves).length) {
                path[j] && "s" === path[j].instruction && makeLonghand(path[j], path[j - 1].data);
                if (hasPrev) {
                  const prevArc = output.shift();
                  roundData(prevArc.data);
                  relSubpoint[0] += prevArc.data[5] - prev.data[prev.data.length - 2];
                  relSubpoint[1] += prevArc.data[6] - prev.data[prev.data.length - 1];
                  prev.instruction = "a";
                  prev.data = prevArc.data;
                  item.base = prev.coords = prevArc.coords;
                }
                arc = output.shift();
                1 === arcCurves.length ? item.sdata = sdata.slice() : arcCurves.length - 1 - hasPrev > 0 && path.splice.apply(path, [ index + 1, arcCurves.length - 1 - hasPrev ].concat(output));
                if (!arc) return !1;
                instruction = "a";
                data = arc.data;
                item.coords = arc.coords;
              }
            }
            if (!1 !== precision) {
              if ("mltqsc".indexOf(instruction) > -1) for (let i = data.length; i--; ) data[i] += item.base[i % 2] - relSubpoint[i % 2]; else if ("h" === instruction) data[0] += item.base[0] - relSubpoint[0]; else if ("v" === instruction) data[0] += item.base[1] - relSubpoint[1]; else if ("a" === instruction) {
                data[5] += item.base[0] - relSubpoint[0];
                data[6] += item.base[1] - relSubpoint[1];
              }
              roundData(data);
              if ("h" === instruction) relSubpoint[0] += data[0]; else if ("v" === instruction) relSubpoint[1] += data[0]; else {
                relSubpoint[0] += data[data.length - 2];
                relSubpoint[1] += data[data.length - 1];
              }
              roundData(relSubpoint);
              if ("m" === instruction.toLowerCase()) {
                pathBase[0] = relSubpoint[0];
                pathBase[1] = relSubpoint[1];
              }
            }
            if (params.straightCurves) if ("c" === instruction && isCurveStraightLine(data) || "s" === instruction && isCurveStraightLine(sdata)) {
              next && "s" === next.instruction && makeLonghand(next, data);
              instruction = "l";
              data = data.slice(-2);
            } else if ("q" === instruction && isCurveStraightLine(data)) {
              next && "t" === next.instruction && makeLonghand(next, data);
              instruction = "l";
              data = data.slice(-2);
            } else if ("t" === instruction && "q" !== prev.instruction && "t" !== prev.instruction) {
              instruction = "l";
              data = data.slice(-2);
            } else if ("a" === instruction && (0 === data[0] || 0 === data[1])) {
              instruction = "l";
              data = data.slice(-2);
            }
            if (params.lineShorthands && "l" === instruction) if (0 === data[1]) {
              instruction = "h";
              data.pop();
            } else if (0 === data[0]) {
              instruction = "v";
              data.shift();
            }
            if (params.collapseRepeated && !hasMarkerMid && "mhv".indexOf(instruction) > -1 && prev.instruction && instruction === prev.instruction.toLowerCase() && ("h" !== instruction && "v" !== instruction || prev.data[0] >= 0 == data[0] >= 0)) {
              prev.data[0] += data[0];
              "h" !== instruction && "v" !== instruction && (prev.data[1] += data[1]);
              prev.coords = item.coords;
              path[index] = prev;
              return !1;
            }
            if (params.curveSmoothShorthands && prev.instruction) if ("c" === instruction) {
              if ("c" === prev.instruction && data[0] === -(prev.data[2] - prev.data[4]) && data[1] === -(prev.data[3] - prev.data[5])) {
                instruction = "s";
                data = data.slice(2);
              } else if ("s" === prev.instruction && data[0] === -(prev.data[0] - prev.data[2]) && data[1] === -(prev.data[1] - prev.data[3])) {
                instruction = "s";
                data = data.slice(2);
              } else if (-1 === "cs".indexOf(prev.instruction) && 0 === data[0] && 0 === data[1]) {
                instruction = "s";
                data = data.slice(2);
              }
            } else if ("q" === instruction) if ("q" === prev.instruction && data[0] === prev.data[2] - prev.data[0] && data[1] === prev.data[3] - prev.data[1]) {
              instruction = "t";
              data = data.slice(2);
            } else if ("t" === prev.instruction && data[2] === prev.data[0] && data[3] === prev.data[1]) {
              instruction = "t";
              data = data.slice(2);
            }
            if (params.removeUseless && !hasStrokeLinecap) {
              if ("lhvqtcs".indexOf(instruction) > -1 && data.every((function(i) {
                return 0 === i;
              }))) {
                path[index] = prev;
                return !1;
              }
              if ("a" === instruction && 0 === data[5] && 0 === data[6]) {
                path[index] = prev;
                return !1;
              }
            }
            item.instruction = instruction;
            item.data = data;
            prev = item;
          } else {
            relSubpoint[0] = pathBase[0];
            relSubpoint[1] = pathBase[1];
            if ("z" === prev.instruction) return !1;
            prev = item;
          }
          return !0;
        }));
      }(data, params);
      params.utilizeAbsolute && (data = function(path, params) {
        let prev = path[0];
        return path.filter((function(item, index) {
          if (0 === index) return !0;
          if (!item.data) {
            prev = item;
            return !0;
          }
          const instruction = item.instruction;
          const data = item.data;
          const adata = data && data.slice(0);
          if ("mltqsc".indexOf(instruction) > -1) for (let i = adata.length; i--; ) adata[i] += item.base[i % 2]; else if ("h" === instruction) adata[0] += item.base[0]; else if ("v" === instruction) adata[0] += item.base[1]; else if ("a" === instruction) {
            adata[5] += item.base[0];
            adata[6] += item.base[1];
          }
          roundData(adata);
          const absoluteDataStr = (0, _tools.cleanupOutData)(adata, params);
          const relativeDataStr = (0, _tools.cleanupOutData)(data, params);
          if (params.forceAbsolutePath || absoluteDataStr.length < relativeDataStr.length && !(params.negativeExtraSpace && instruction === prev.instruction && prev.instruction.charCodeAt(0) > 96 && absoluteDataStr.length === relativeDataStr.length - 1 && (data[0] < 0 || /^0\./.test(data[0]) && prev.data[prev.data.length - 1] % 1))) {
            item.instruction = instruction.toUpperCase();
            item.data = adata;
          }
          prev = item;
          return !0;
        }));
      }(data, params));
      (0, _path.js2path)(item, data, params);
    }
  }
};

function isConvex(data) {
  const center = getIntersection([ 0, 0, data[2], data[3], data[0], data[1], data[4], data[5] ]);
  return center && data[2] < center[0] == center[0] < 0 && data[3] < center[1] == center[1] < 0 && data[4] < center[0] == center[0] < data[0] && data[5] < center[1] == center[1] < data[1];
}

function getIntersection(coords) {
  const a1 = coords[1] - coords[3];
  const b1 = coords[2] - coords[0];
  const c1 = coords[0] * coords[3] - coords[2] * coords[1];
  const a2 = coords[5] - coords[7];
  const b2 = coords[6] - coords[4];
  const c2 = coords[4] * coords[7] - coords[5] * coords[6];
  const denom = a1 * b2 - a2 * b1;
  if (!denom) return;
  const cross = [ (b1 * c2 - b2 * c1) / denom, (a1 * c2 - a2 * c1) / -denom ];
  return !isNaN(cross[0]) && !isNaN(cross[1]) && isFinite(cross[0]) && isFinite(cross[1]) ? cross : void 0;
}

function strongRound(data) {
  for (let i = data.length; i-- > 0; ) if (data[i].toFixed(precision) !== data[i]) {
    const rounded = +data[i].toFixed(precision - 1);
    data[i] = +Math.abs(rounded - data[i]).toFixed(precision + 1) >= error ? +data[i].toFixed(precision) : rounded;
  }
  return data;
}

function round(data) {
  for (let i = data.length; i-- > 0; ) data[i] = Math.round(data[i]);
  return data;
}

function isCurveStraightLine(data) {
  let i = data.length - 2;
  const a = -data[i + 1];
  const b = data[i];
  const d = 1 / (a * a + b * b);
  if (i <= 1 || !isFinite(d)) return !1;
  for (;(i -= 2) >= 0; ) if (Math.sqrt(Math.pow(a * data[i] + b * data[i + 1], 2) * d) > error) return !1;
  return !0;
}

function makeLonghand(item, data) {
  switch (item.instruction) {
   case "s":
    item.instruction = "c";
    break;

   case "t":
    item.instruction = "q";
  }
  item.data.unshift(data[data.length - 2] - data[data.length - 4], data[data.length - 1] - data[data.length - 3]);
  return item;
}

function getDistance(point1, point2) {
  return Math.hypot(point1[0] - point2[0], point1[1] - point2[1]);
}

function getCubicBezierPoint(curve, t) {
  const sqrT = t * t;
  const cubT = sqrT * t;
  const mt = 1 - t;
  const sqrMt = mt * mt;
  return [ 3 * sqrMt * t * curve[0] + 3 * mt * sqrT * curve[2] + cubT * curve[4], 3 * sqrMt * t * curve[1] + 3 * mt * sqrT * curve[3] + cubT * curve[5] ];
}

function isArc(curve, circle) {
  const tolerance = Math.min(arcThreshold * error, arcTolerance * circle.radius / 100);
  return [ 0, 1 / 4, .5, 3 / 4, 1 ].every((function(point) {
    return Math.abs(getDistance(getCubicBezierPoint(curve, point), circle.center) - circle.radius) <= tolerance;
  }));
}

function isArcPrev(curve, circle) {
  return isArc(curve, {
    center: [ circle.center[0] + curve[4], circle.center[1] + curve[5] ],
    radius: circle.radius
  });
}

function findArcAngle(curve, relCircle) {
  const x1 = -relCircle.center[0];
  const y1 = -relCircle.center[1];
  const x2 = curve[4] - relCircle.center[0];
  const y2 = curve[5] - relCircle.center[1];
  return Math.acos((x1 * x2 + y1 * y2) / Math.sqrt((x1 * x1 + y1 * y1) * (x2 * x2 + y2 * y2)));
}

function data2Path(params, pathData) {
  return pathData.reduce((function(pathString, item) {
    let strData = "";
    item.data && (strData = (0, _tools.cleanupOutData)(roundData(item.data.slice()), params));
    return pathString + item.instruction + strData;
  }), "");
}