"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.fn = exports.params = exports.description = exports.active = exports.type = exports.name = void 0;

var _tools = require("../lib/svgo/tools");

var _transforms = require("./_transforms");

exports.name = "convertTransform";

exports.type = "perItem";

exports.active = !0;

exports.description = "collapses multiple transformations and optimizes it";

exports.params = {
  convertToShorts: !0,
  floatPrecision: 3,
  transformPrecision: 5,
  matrixToTransform: !0,
  shortTranslate: !0,
  shortScale: !0,
  shortRotate: !0,
  removeUseless: !0,
  collapseIntoOne: !0,
  leadingZero: !0,
  negativeExtraSpace: !1
};

let degRound;

let floatRound;

let transformRound;

exports.fn = function(item, params) {
  if (item.elem) {
    item.hasAttr("transform") && convertTransform(item, "transform", params);
    item.hasAttr("gradientTransform") && convertTransform(item, "gradientTransform", params);
    item.hasAttr("patternTransform") && convertTransform(item, "patternTransform", params);
  }
};

function convertTransform(item, attrName, params) {
  let data = (0, _transforms.transform2js)(item.attr(attrName).value);
  (params = function(data, params) {
    const matrixData = data.reduce(getMatrixData, []);
    let significantDigits = params.transformPrecision;
    params = Object.assign({}, params);
    if (matrixData.length) {
      params.transformPrecision = Math.min(params.transformPrecision, Math.max.apply(Math, matrixData.map(floatDigits)) || params.transformPrecision);
      significantDigits = Math.max.apply(Math, matrixData.map((function(n) {
        return String(n).replace(/\D+/g, "").length;
      })));
    }
    "degPrecision" in params || (params.degPrecision = Math.max(0, Math.min(params.floatPrecision, significantDigits - 2)));
    floatRound = params.floatPrecision >= 1 && params.floatPrecision < 20 ? smartRound.bind(this, params.floatPrecision) : round;
    degRound = params.degPrecision >= 1 && params.floatPrecision < 20 ? smartRound.bind(this, params.degPrecision) : round;
    transformRound = params.transformPrecision >= 1 && params.floatPrecision < 20 ? smartRound.bind(this, params.transformPrecision) : round;
    return params;
  }(data, params)).collapseIntoOne && data.length > 1 && (data = [ (0, _transforms.transformsMultiply)(data) ]);
  params.convertToShorts ? data = function(transforms, params) {
    for (let i = 0; i < transforms.length; i++) {
      let transform = transforms[i];
      if (params.matrixToTransform && "matrix" === transform.name) {
        const decomposed = (0, _transforms.matrixToTransform)(transform, params);
        decomposed !== transform && js2transform(decomposed, params).length <= js2transform([ transform ], params).length && transforms.splice.apply(transforms, [ i, 1 ].concat(decomposed));
        transform = transforms[i];
      }
      roundTransform(transform);
      params.shortTranslate && "translate" === transform.name && 2 === transform.data.length && !transform.data[1] && transform.data.pop();
      params.shortScale && "scale" === transform.name && 2 === transform.data.length && transform.data[0] === transform.data[1] && transform.data.pop();
      if (params.shortRotate && transforms[i - 2] && "translate" === transforms[i - 2].name && "rotate" === transforms[i - 1].name && "translate" === transforms[i].name && transforms[i - 2].data[0] === -transforms[i].data[0] && transforms[i - 2].data[1] === -transforms[i].data[1]) {
        transforms.splice(i - 2, 3, {
          name: "rotate",
          data: [ transforms[i - 1].data[0], transforms[i - 2].data[0], transforms[i - 2].data[1] ]
        });
        i -= 2;
        transform = transforms[i];
      }
    }
    return transforms;
  }(data, params) : data.forEach(roundTransform);
  params.removeUseless && (data = data.filter((function(transform) {
    return !([ "translate", "rotate", "skewX", "skewY" ].indexOf(transform.name) > -1 && (1 === transform.data.length || "rotate" === transform.name) && !transform.data[0] || "translate" === transform.name && !transform.data[0] && !transform.data[1] || "scale" === transform.name && 1 === transform.data[0] && (transform.data.length < 2 || 1 === transform.data[1]) || "matrix" === transform.name && 1 === transform.data[0] && 1 === transform.data[3] && !(transform.data[1] || transform.data[2] || transform.data[4] || transform.data[5]));
  })));
  data.length ? item.attr(attrName).value = js2transform(data, params) : item.removeAttr(attrName);
}

function getMatrixData(a, b) {
  return "matrix" === b.name ? a.concat(b.data.slice(0, 4)) : a;
}

function floatDigits(n) {
  return (n = String(n)).slice(n.indexOf(".")).length - 1;
}

function js2transform(transformJS, params) {
  let transformString = "";
  transformJS.forEach((function(transform) {
    roundTransform(transform);
    transformString += (transformString && " ") + transform.name + "(" + (0, _tools.cleanupOutData)(transform.data, params) + ")";
  }));
  return transformString;
}

function roundTransform(transform) {
  switch (transform.name) {
   case "translate":
    transform.data = floatRound(transform.data);
    break;

   case "rotate":
    transform.data = degRound(transform.data.slice(0, 1)).concat(floatRound(transform.data.slice(1)));
    break;

   case "skewX":
   case "skewY":
    transform.data = degRound(transform.data);
    break;

   case "scale":
    transform.data = transformRound(transform.data);
    break;

   case "matrix":
    transform.data = transformRound(transform.data.slice(0, 4)).concat(floatRound(transform.data.slice(4)));
  }
  return transform;
}

function round(data) {
  return data.map(Math.round);
}

function smartRound(precision, data) {
  for (let i = data.length, tolerance = +Math.pow(.1, precision).toFixed(precision); i--; ) if (data[i].toFixed(precision) !== data[i]) {
    const rounded = +data[i].toFixed(precision - 1);
    data[i] = +Math.abs(rounded - data[i]).toFixed(precision + 1) >= tolerance ? +data[i].toFixed(precision) : rounded;
  }
  return data;
}