"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

exports.fn = exports.params = exports.description = exports.active = exports.type = exports.name = void 0;

var _cssTree = (obj = require("css-tree")) && obj.__esModule ? obj : {
  default: obj
};

var obj;

var _collections = require("./_collections.js");

exports.name = "prefixIds";

exports.type = "perItem";

exports.active = !1;

exports.params = {
  delim: "__",
  prefixIds: !0,
  prefixClassNames: !0
};

exports.description = "prefix IDs";

const rxId = /^#(.*)$/;

let addPrefix = null;

const attrNotEmpty = function(attr) {
  return attr && attr.value && attr.value.length > 0;
};

const prefixId = function(val) {
  const idName = function(urlVal) {
    const idUrlMatches = urlVal.match(rxId);
    return null !== idUrlMatches && idUrlMatches[1];
  }(val);
  return !!idName && "#" + addPrefix(idName);
};

const addPrefixToHrefAttr = function(attr) {
  if (!attrNotEmpty(attr)) return;
  const idPrefixed = prefixId(attr.value);
  idPrefixed && (attr.value = idPrefixed);
};

const addPrefixToUrlAttr = function(attr) {
  if (!attrNotEmpty(attr)) return;
  const urlVal = function(val) {
    const urlMatches = /url\((.*?)\)/gi.exec(val);
    return null !== urlMatches && urlMatches[1];
  }(attr.value);
  if (!urlVal) return;
  const idPrefixed = prefixId(urlVal);
  idPrefixed && (attr.value = "url(" + idPrefixed + ")");
};

const addPrefixToBeginEndAttr = function(attr) {
  if (!attrNotEmpty(attr)) return;
  const parts = attr.value.split("; ").map((function(val) {
    if ((val = val.trim()).endsWith(".end") || val.endsWith(".start")) {
      const idPostfix = val.split(".");
      const id = idPostfix[0];
      const postfix = idPostfix[1];
      let idPrefixed = prefixId(`#${id}`);
      if (!idPrefixed) return val;
      idPrefixed = idPrefixed.slice(1);
      return `${idPrefixed}.${postfix}`;
    }
    return val;
  }));
  attr.value = parts.join("; ");
};

exports.fn = function(node, opts, extra) {
  if (extra.multipassCount && extra.multipassCount > 0) return node;
  let prefix = "prefix";
  opts.prefix ? prefix = "function" == typeof opts.prefix ? opts.prefix(node, extra) : opts.prefix : !1 === opts.prefix ? prefix = !1 : extra && extra.path && extra.path.length > 0 && (prefix = extra.path.split(/[/\\]/).pop());
  addPrefix = function(name) {
    return (!1 === prefix ? name : prefix + opts.delim + name).replace(/[. ]/g, "_");
  };
  if ("style" === node.elem) {
    if (node.isEmpty()) return node;
    const cssStr = node.content[0].text || node.content[0].cdata || [];
    let cssAst = {};
    try {
      cssAst = _cssTree.default.parse(cssStr, {
        parseValue: !0,
        parseCustomProperty: !1
      });
    } catch (parseError) {
      console.warn("Warning: Parse error of styles of <style/> element, skipped. Error details: " + parseError);
      return node;
    }
    let idPrefixed = "";
    _cssTree.default.walk(cssAst, (function(node) {
      if ((opts.prefixIds && "IdSelector" === node.type || opts.prefixClassNames && "ClassSelector" === node.type) && node.name) node.name = addPrefix(node.name); else if ("Url" === node.type && node.value.value && node.value.value.length > 0) {
        idPrefixed = prefixId(((string = "") => {
          const {length} = string;
          if (0 !== length) {
            const firstCharCode = string.charCodeAt(0);
            const lastCharCode = string.charCodeAt(length - 1);
            const dropFirst = 39 === firstCharCode || 34 === firstCharCode;
            const dropLast = 39 === lastCharCode || 34 === lastCharCode;
            string = string.slice(dropFirst ? 1 : 0, dropLast ? length - 1 : length);
          }
          return string;
        })(node.value.value));
        if (!idPrefixed) return;
        node.value.value = idPrefixed;
      }
    }));
    node.content[0].text = _cssTree.default.generate(cssAst);
    return node;
  }
  if (!node.attrs) return node;
  opts.prefixIds && (attr = node.attrs.id, attrNotEmpty(attr) && (attr.value = addPrefix(attr.value)));
  var attr;
  opts.prefixClassNames && function(attr) {
    attrNotEmpty(attr) && (attr.value = attr.value.split(/\s+/).map(addPrefix).join(" "));
  }(node.attrs.class);
  addPrefixToHrefAttr(node.attrs.href);
  addPrefixToHrefAttr(node.attrs["xlink:href"]);
  for (const referencesProp of _collections.referencesProps) addPrefixToUrlAttr(node.attrs[referencesProp]);
  addPrefixToBeginEndAttr(node.attrs.begin);
  addPrefixToBeginEndAttr(node.attrs.end);
  return node;
};